# @omlish-lite
"""
Jobs:
 - globals
 - pkgs
 - pyenv
 - tailscale
 - docker
 - system nginx
 - system service manager - systemd / supervisor
 - users
 - firewall

Deploy:
 - User
 - Dirs
 - GlobalNginx
 - GlobalSupervisor
 - Repo
 - Venv
 - Supervisor
 - Nginx
"""
